from . import add
from . import check_migrate
from . import list
from . import migrate
from . import rename
from . import rm
from . import show
from . import update

commands_list = [
    add,
    check_migrate,
    list,
    migrate,
    rename,
    rm,
    show,
    update
]
